#include <stdio.h>
#include <string.h>    //strlen
#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include <unistd.h>
#include "esp_event_loop.h"
#include "freertos/event_groups.h"
#include "esp_system.h"
#include "esp_log.h"
#include "lwip/err.h"
#include "lwip/sockets.h"
#include "lwip/sys.h"
#include "lwip/netdb.h"
#include "lwip/dns.h"
#include <arpa/inet.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <sys/socket.h>

#include "udp_manager.h"

static const char *TAG = "UDP_MANAGER";

int16_t udp_port = 0;

TaskHandle_t udp_task_handle = NULL;

struct sockaddr_in cliaddr;

int udp_server_socket;
static unsigned int socklen;

bool udp_ready = false;

bool is_udp_ready()
{
    return udp_ready;
}

static int udp_send_data(ip4_addr_t ip ,char* buf, size_t buf_len)
{
    cliaddr.sin_addr.s_addr = ip.addr;
    return sendto(udp_server_socket, (const char *)buf, buf_len, 0, (const struct sockaddr *) &cliaddr, socklen);;
}

int udp_send_data_blocking(ip4_addr_t ip ,char* buf, size_t buf_len, uint32_t timeout_ms)
{
    int64_t start = esp_timer_get_time()/1000;
    int64_t now = start;

    int err_code = 0;
    int32_t actual_send = 0;
    bool timeout = false;

    int tries = 0;

    do
    {
        actual_send = udp_send_data(ip, buf, buf_len);
        tries++;

        err_code = (actual_send < 0) ? errno : 0;

        now = esp_timer_get_time()/1000;
        timeout = (now-start) > timeout_ms;

        if(!err_code || (err_code != ENOMEM))
            break;

        vTaskDelay(1);

    }while(!timeout);

    if(timeout || err_code)
        ESP_LOGE(TAG, "SENDTO - %s (%d) (timeout: %d, socket: %d, ip_addr: %s, socklen: %d)", strerror(err_code), err_code, timeout, udp_server_socket, ip4addr_ntoa(&ip), socklen);

    if((tries > 1) || (actual_send != buf_len))
        ESP_LOGW(TAG, "SENDTO in %d tries and %d ms (size:%d, buf_len:%d)", tries, (uint32_t)(now-start), actual_send, buf_len);

    return (timeout || err_code || (actual_send != buf_len) || (tries > 1));
}

void udp_stop()
{
    udp_ready = false;

    close(udp_server_socket);
}

void udp_start()
{
    cliaddr.sin_addr.s_addr = inet_addr("192.168.4.2");
    cliaddr.sin_family = AF_INET;
    cliaddr.sin_port = htons( udp_port );
    socklen = sizeof(cliaddr);

    udp_server_socket = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
    if(udp_server_socket < 0)
    {
        ESP_LOGE(TAG, "... Failed to allocate socket.\n");
        return;
    }
    ESP_LOGI(TAG, "... allocated socket\n");

    int opt = 0;
    setsockopt(udp_server_socket, SOL_SOCKET, SO_REUSEADDR, &opt, sizeof(opt));

    udp_ready = true;
}

void udp_manager_init()
{
    udp_port = UDP_DEFAULT_PORT;
    udp_start();
}