#ifndef WIFI_MANAGER_H_
#define WIFI_MANAGER_H_

#include <esp_wifi.h>

#define ESP_MAX_STA_CONN 2
#define MAC_ADDRESS_SIZE 6

typedef enum
{
    WIFI_STA_IS_DISCONNECTED = 0,
    WIFI_STA_IS_CONNECTED = 1,
    WIFI_STA_IS_CONNECTED_WITH_IP = 2,
}wifi_state_code;

typedef struct sta_connected_data
{
    ip4_addr_t sta_ip;
    uint8_t sta_mac[6];
    uint16_t sta_aid;
    int64_t connected_time;
}sta_connected_data_t;

typedef struct wifi_config_param
{
    char ap_ssid[32];
    char ap_pwd[64];
    uint8_t ap_bssid[6];
    int8_t tx_power;
    wifi_bandwidth_t bandwidth;
    uint8_t primary_channel;
    wifi_second_chan_t secondary_channel;
    sta_connected_data_t *connected_sta;
}wifi_config_param_t;

typedef esp_err_t (*wifi_event_cb)(system_event_t* event_id);

void wifi_manager_init(wifi_event_cb fct_cb, char* ap_ssid, char* ap_pwd);

char* get_auth_type_string(wifi_auth_mode_t auth_idx);
esp_err_t wifi_get_ap_params(char* new_ap_ssid, char* new_ap_pwd);
esp_err_t wifi_set_ap_params(char* new_ap_ssid, char* new_ap_pwd);
wifi_config_t wifi_get_config();
esp_err_t wifi_enable_interface(bool wifi_enable);
esp_err_t wifi_ap_set_bw_and_channel(wifi_bandwidth_t wifi_bw, uint8_t new_primary_channel, wifi_second_chan_t new_secondary_channel);
esp_err_t wifi_ap_set_tx_power(int8_t new_tx_power);
wifi_config_param_t* get_wifi_base_param();
#endif
